@echo off
set COPYCMD=/y
REM
REM Copyright (c) 2001,2003 by Oracle Corporation. All Rights Reserved. \
REM
REM
REM OHS mod_plsql/OWA Patch Driver Script
REM
REM This script applies mod_plsql and OWA package patches 
REM Invoke "patch.cmd" for help
REM

REM
REM Initialize default argument values
REM
set sys_schema=sys
set sys_pwd=change_on_install
set conn_str=""
set patch_midtier=""
set patch_owa=""
set logf=patch.log
set ohome=""
set bkpname=prepatch
set log_fname=%logf%

REM
REM Show the usage if no arguments were passed
REM
set return_code=0
if (%1)==() goto usage

REM
REM Parse the arguments
REM

:STEP1
if "%1" == "" goto STEP2

:ARG1
if not "%1" == "-o" goto ARG2
shift
set ohome=%1
shift
goto STEP1

:ARG2
if not "%1" == "-m" goto ARG3
shift
if not "%1" == "midtier" goto VALIDATE_MODE
goto MODE_OK

:VALIDATE_MODE
if "%1" == "sysobjects" goto MODE_OK
 echo.
 echo Script Failed
 echo -------------
 echo Error  : Invalid option '%1' for patch mode
 echo Cause  : You have specified an invalid mode string
 echo Action : Pass a valid -m option (midtier or sysobjects)
 echo.
goto THE_END

:MODE_OK
if "%1" == "midtier" (
   set patch_midtier=%1
)

if "%1" == "sysobjects" (
   set patch_owa=%1
)
shift
goto STEP1

:ARG3
if not "%1" == "-p" goto ARG4
shift
set sys_pwd=%1
shift
goto STEP1

:ARG4
if not "%1" == "-c" goto ARG5
shift
set conn_str=%1
shift
goto STEP1

:ARG5
if not "%1" == "-l" goto ARG6
shift
set log_fname=%1
shift
goto STEP1

:ARG6
if not "%1" == "-h" goto WRONG_MODE
goto usage

:STEP2
REM
REM wipe out the contents of patch log
echo. > %log_fname%
REM
REM Is the user in the right directory?
REM
if exist "patch.cmd" goto FILE1
   echo.
   echo Script Failed
   echo -------------
   echo Error  : Invalid current directory for running the patch script
   echo Cause  : You have tried to execute the patch script remotely
   echo Action : Change your current directory to match the location of
   echo          the patch script
   echo.
   goto THE_END

:FILE1

REM
REM Does this location have the required files (or at least some of them)
REM
if  exist "modplsql\bin" goto FILE2
   echo.
   echo Script Failed
   echo -------------
   echo Error  : Patch contents cannot be found
   echo Cause  : Patch contents do not exist in the current directory
   echo Action : Download the patch again, or check for missing files
   echo.
   goto THE_END
:FILE2

REM
REM Verify that user specified the mode for the install
REM
if %patch_midtier% == "" goto VALIDATE_PATCH
goto VALID_PATCH

:VALIDATE_PATCH
if not %patch_owa% == "" goto VALID_PATCH
   echo.
   echo Script Failed
   echo -------------
   echo Error  : Patch script invoked without a mode
   echo Cause  : You did not specify the -m option
   echo Action : Re-run the patch script with a valid -m option
   echo.
   goto THE_END

:VALID_PATCH

REM
REM Check Oracle home argument
REM
REM Is ORACLE_HOME passed in?
if not "%ohome%" == "" goto OH_SET
   echo.
   echo Script Failed
   echo -------------
   echo Error  : Oracle Home argument not specified
   echo Cause  : You have not specified the -o option
   echo Action : Pass a valid -o option for the correct ORACLE_HOME
   echo.
   goto THE_END

:OH_SET

REM
REM Check oracle home is valid
REM
set oh_modplsql=%ohome%\Apache\modplsql
if exist "%oh_modplsql%" goto OH_MODPLSQL
   echo.
   echo Script Failed
   echo -------------
   echo Error  : Oracle Home %ohome% is invalid. %oh_modplsql% not found
   echo Cause  : You have specified a non-existent Oracle Home argument
   echo Action : Pass a valid -o option for the correct ORACLE_HOME
   echo.
   goto THE_END

:OH_MODPLSQL

REM
REM Set platform specific values
REM
set lib_dir=%ohome%\bin
set modplsql_binary=%ohome%\bin\modplsql.dll

REM
REM Is the ORACLE_HOME argument valid?
REM
if  exist "%lib_dir%" goto LIB_EXISTS
   echo.
   echo Script Failed
   echo -------------
   echo Error  : Oracle Home %ohome% is invalid. %lib_dir% not found
   echo Cause  : You have specified a non-existent Oracle Home argument
   echo Action : Pass a valid -o option for the correct ORACLE_HOME
   echo.
   goto THE_END

:LIB_EXISTS

REM
REM Validate parameters needed for patching OWA packages
REM
if "%patch_owa%" == "sysobjects" goto VALIDATE_PARM
goto PARM_OK

:VALIDATE_PARM
if not %conn_str% == "" goto PARM_OK
   echo.
   echo Script Failed
   echo -------------
   echo Error  : Database connect string not specified
   echo Cause  : You have not specified the -c option
   echo Action : Pass a valid -c option for the database connect string
   echo.
   goto THE_END

:PARM_OK

REM
REM Setup environment for running SQL*Plus, if needed
REM
if "%patch_owa%" == "sysobjects" goto PATCH_OWA

REM
REM Validate parameters needed for patching mod_plsql
REM
:MIDTIER
if "%patch_midtier%" == "midtier" goto PATCH_MIDTIER
goto WRITE_LOG


:PATCH_OWA
REM
REM Setup Path, on windows this contains both DLLs and binaries
REM
if not "%PATH%" == ""  set LIBPATH=%ohome%\bin & goto PATH_SET
set PATH=%ohome%\bin;%PATH%

:PATH_SET

REM
REM Validate connect string passed down by the end user
REM
REM Validate Connect String
sqlplus /nolog @valcstr.sql %sys_schema% %sys_pwd% %conn_str%  >> %log_fname%
set return_code=%errorlevel%
if "%return_code%" == "0" goto MIDTIER
   echo.
   echo Script Failed
   echo -------------
   echo Error  : DB connection failed using %sys_schema%/%sys_pwd%@%conn_str%
   echo Cause  : Failed to connect using SYS as sysdba to the database
   echo Action : Check the following details
   echo          - %sys_schema% password is correct
   echo          - Database connect string information is correct
   echo          - Database connect string can be resolved by Net
   echo          - Oracle Database/TNS Listener are up
   echo          - SQL*Plus can successfully invoke the following command
   echo               SQL^> conn %sys_schema%/******@%conn_str% as sysdba
   echo          - Database is configured to allow remote connections as
   echo            sysdba. Invoke patch.cmd with the -h option for more
   echo            information on when the database will not allow remote
   echo            connections, and what can be done
   echo.
   goto THE_END

REM end of PATCH_OWA

:PATCH_MIDTIER
REM
REM Does the ORACLE_HOME have mod_plsql installed in it?
REM
if  exist "%modplsql_binary%" goto MODBIN_EXISTS
   echo.
   echo Script Failed
   echo -------------
   echo Error  : Oracle Home invalid. %modplsql_binary% not found
   echo Cause  : You have specified a non-existent Oracle Home argument
   echo Action : Pass a valid -o option for the correct ORACLE_HOME
   echo.
   goto THE_END

:MODBIN_EXISTS

REM
REM Does a backup directory already exist
REM
set bkpdir=%ohome%\Apache\modplsql\%bkpname%
if not exist "%bkpdir%" goto MAKE_BKPDIR
   echo.
   echo Script Failed
   echo -------------
   echo Error  : Backup directory %bkpdir% already exists
   echo Cause  : The patch detected a previous backup directory
   echo Action : Remove or archive the existing backup directory
   echo.
   goto THE_END

:MAKE_BKPDIR

REM
REM Create the required directories to backup stuff, and do the backup
REM
mkdir %bkpdir%
mkdir %bkpdir%\bin

xcopy /S %ohome%\bin\modplsql.dll %bkpdir%\bin\
if "%errorlevel%" == "0" goto MAKE_CFGDIR
goto MAKE_CONFDIR

:MAKE_CFGDIR
if exist modplsql\cfg    mkdir %bkpdir%\cfg & xcopy /S %ohome%\Apache\modplsql\cfg\* %bkpdir%\cfg\


:MAKE_CONFDIR
if exist modplsql\conf    mkdir %bkpdir%\conf & xcopy /S %ohome%\Apache\modplsql\conf\* %bkpdir%\conf\

REM
REM Did the backup succeed?
REM
if "%errorlevel%" == "0" goto WRITE_LOG
   set return_code=1
   echo Script Failed
   echo -------------
   echo Error  : Failed to backup original files to %bkpdir%
   echo Cause  : Possible read/write privileges
   echo Action : Make sure that the privileges exist for the user
   echo.
   del/Q %bkpdir%
   goto THE_END

REM end of PATCH_MIDTIER fallback to WRITE_LOG

:WRITE_LOG
echo.
echo. >> %log_fname%

echo Welcome to mod_plsql/OWA Patch
echo ==============================
echo Welcome to mod_plsql/OWA Patch >> %log_fname%
echo ============================== >> %log_fname%

echo. >> %log_fname%
echo The following details have been determined : >> %log_fname%
echo. >> %log_fname%
echo Patch Mode      : %patch_midtier% %patch_owa% >> %log_fname%
echo ORACLE_HOME     : %ohome% >> %log_fname%
REM echo PATH            : '%PATH%' >> %log_fname%
if not "%PATH%" == ""   echo PATH            : %PATH% >> %log_fname%
if  "%patch_owa%" == "sysobjects" echo Connect String  : %conn_str% >> %log_fname%
echo Log File Name   : %log_fname% >> %log_fname%
echo. >> %log_fname%
echo. >> %log_fname%
echo Patch Started at 
date /t
echo Patch Started at >> %log_fname%
date /t >> %log_fname%
echo =========================================== >> %log_fname%


REM
REM Load the new OWA packages
REM
if  not "%patch_owa%" == "sysobjects" goto P_OWA1
   echo ...Installing OWA packages using script patload_win.sql
   echo ...Installing OWA packages using script patload_win.sql >> %log_fname%
   sqlplus /nolog @patload_win.sql %sys_schema% %sys_pwd% %conn_str%  >> %log_fname%
   set return_code=%errorlevel%
   if "%return_code%" == "0" goto P_OWA1
   echo Error  : You are possibly running a pre-8.1.7.4 version of the database >> %log_fname%
   echo Action : Upgrade the database to 8.1.7.4 or higher >> %log_fname%
   echo  >> %log_fname%
   
:P_OWA1

if not "%return_code%" == "0" goto finish

REM
REM Patch the midtier
REM
if not "%patch_midtier%" == "midtier" goto P_MTIER
   echo Backup done for files to %bkpdir%
   echo Backup done for files to %bkpdir% >> %log_fname%
   echo ...Installing mod_plsql patch
   echo ...Installing mod_plsql patch >> %log_fname%

REM 
REM    Overlay newer bits on top of the older bits
REM 

REM   Patch up modplsql
   xcopy /S modplsql\* %ohome%\Apache\modplsql
   set return_code=%errorlevel%
   copy %ohome%\Apache\modplsql\bin\modplsql.dll %ohome%\bin\
   set return_code=%errorlevel%

if "%return_code%" == "0" goto FINISH1
   echo    Failed to patch %ohome%\Apache\modplsql\* >> %log_fname%
   echo    Failed to patch %ohome%\Apache\modplsql\*
   goto finish


:FINISH1
echo    Successful copy to %ohome%\Apache\modplsql\*
echo    Successful copy to %ohome%\Apache\modplsql\* >> %log_fname%
echo    Successful copy of modplsql.dll to %ohome%\bin
echo    Successful copy of modplsql.dll to %ohome%\\bin >> %log_fname%

:P_MTIER

:FINISH
echo Patch Ended at 
date /t
echo Patch Ended at >> %log_fname%
date /t >> %log_fname%

set RESULT=" Successfully "
if not "%return_code%" == "0"    set RESULT=" With Error(s) "

echo.
echo. >> %log_fname%
echo  **Patch Completed %RESULT% **.  >> %log_fname%
echo  **Patch Completed %RESULT% **.  
echo Scan log %log_fname% for any errors.
echo.

REM end of patch
goto THE_END

:USAGE
echo.
echo Usage^: patch.cmd ^[-h^] -m ^<patch_mode^> ^[-l ^<log_file_name^>^]
echo                  ^[-p ^<sys_password^>^] ^[-c ^<connect_string^>^]
echo where
echo.
echo     -m ^<patch_mode^> specifies the patch_mode and can take values of
echo         midtier    : patches mod_plsql in the middle-tier
echo         sysobjects : patches the OWA packages in the database
echo.
echo     -l ^<log_file_name^> specifies the log filename. Default=patch.log
echo.
echo     -h brings up this help screen
echo.
echo     For sysobjects mode, the following arguments are additionally needed
echo.
echo     -p ^<sys_password^> is the SYS schema password.Default=change_on_install
echo.
echo     -c ^<connect_string^> is the Database connect string
echo.
echo Please view README.TXT for instructions for this particular patch
echo. 
echo How to Invoke:
echo 1. You can patch mod_plsql and OWA by running
echo     ^> patch.cmd -m midtier -m sysobjects -p syspassword -c connect_string
echo. 
echo 2. You can just patch mod_plsql by running
echo     ^> patch.cmd -m midtier
echo. 
echo 3. You can just patch OWA packages by running
echo     ^> patch.cmd -m sysobjects -p syspassword -c connectstring
echo. 
echo  Optionally, you can use -l to specify a log file other than patch.log
echo. 
echo Notes :
echo - For midtier mode, Oracle HTTP Server must be shutdown before installing
echo   the patch
echo.
echo - For sysobjects mode, the following Oracle database parameter settings
echo   could disallow remote database connections as sysdba
echo      * O7_DICTIONARY_ACCESSIBILITY = FALSE
echo      * REMOTE_LOGIN_ PASSWORDFILE = NONE
echo   If this is the case, either allow remote connections temporarily, or
echo   upload the patch to the database machine and run the patch script in
echo   sysobjects mode on the database machine
echo.
echo   For more details on these Oracle parameters, refer to the Oracle
echo   documentation.
echo.
echo - This script will backup older data to ^<OH^>\Apache\modplsql\%bkpname%
echo   A directory with the same name should not pre-exist
echo.
goto THE_END

:WRONG_MODE
echo.
echo Script Failed
echo -------------
echo Error  : Unknown option %1
echo Cause  : You have specified an unknown option
echo Action : Remove or Fix the invalid option
echo.
goto THE_END

:THE_END
echo.
